(function ( $ ) {
    var oldie = [];
    var delay = 10;
    var effect;
    var effects=[];

    /**
     * Check if an object has a given property
     *
     * @param obj
     * @param prop
     * @returns {boolean}
     */
    function hasOwnProperty(obj, prop) {
        var proto = obj.__proto__ || obj.constructor.prototype;
        return (prop in obj) &&
            (!(prop in proto) || proto[prop] !== obj[prop]);
    }
    /**
     * Format the inserted effect to match the animate.css effect
     *
     * @param effect;
     * @return string
     */
    var getEffect = function(effect){
        var splitted = effect.split(" ");
        var first=true;
        effect = "";
        var x;
        for(x in splitted)
        {
            if(first){
                effect += splitted[x];
                first=false;
            }else{
                effect += String(splitted[x].charAt(0).toUpperCase() + splitted[x].slice(1));
            }
        }
        return effect;
    }
    /**
     * Do custom things after the effect ends.
     *
     * @param jquery object
     */
    var doDelayThings = function(options){
        if(typeof options === 'object' && hasOwnProperty(options, "delay")){
            if(options['delay'] !== null){
                delay = parseInt(options['delay']);
            }else {
                delay = 10;
            }
        }else{
            delay=10;
        }
    }
    /**
     * Do some checks on the passed parameter
     *
     * @param options
     */
    var doTheChecks = function( options ){
        //check if the user defined an object
        if(options !== null && typeof options === 'object'){
            //check if user defined an effect property
            if(hasOwnProperty(options, "effect")){
                effect = getEffect(options['effect']);
            }
            //else set a default effect
            else{
                effect = 'fadeIn';
            }
            effects[effect]={};
            //check if user defined onStart property
            if(hasOwnProperty(options, "onStart")){
                effects[effect]['onStart'] = options['onStart'];
            }else{
                effects[effect]['onStart'] = 'show';
            }
            //check if user defined onEnd property
            if(hasOwnProperty(options, "onEnd")){
                effects[effect]['onEnd'] = options['onEnd'];
            }else{
                effects[effect]['onEnd'] = '';
            }
            //if the effect is an 'Out' => set the onEnd callback to hide.
            if(effect.search("Out")>=0){
                effects[effect]['onEnd'] = 'hide';
            }
        }
        //else check if the user inserted a string
        else if(options !== null){
            effect = getEffect(options);
            effects[effect]={};
            if(effect.search("Out")>=0){
                effects[effect]['onEnd'] = 'hide';
                effects[effect]['onStart'] = '';
            }else{
                effects[effect]['onStart'] = 'show';
                effects[effect]['onEnd'] = '';
            }
        }
        //else set a default effect
        else{
            effect = 'fadeIn';
            effects[effect]['onStart'] = 'show';
            effects[effect]['onEnd'] = '';
        }
    }
    /**
     * Check and execute the onStart parameter
     *
     * @param kwix
     */
    var doBeforeThings = function(kwix){
        //get current effect's onStart property
        var onStart = effects[effect]['onStart'];

        //
        switch(onStart){
            case 'show':
                kwix.show();
                break;
            default:
                if(typeof onStart == 'function'){
                    onStart.call(this);
                }
                break;
        }
    }
    /**
     * Eliminate the old effect (if exists) and add the new one
     *
     * @param kwix
     */
    var doTheThings = function( kwix ){
        if(oldie[kwix.selector] !== undefined && oldie[kwix.selector] !== null){
            kwix.removeClass("animated "+oldie[kwix.selector]);
        }
        kwix.addClass("animated "+effect);
        oldie[kwix.selector] = effect;
    }
    /**
     * Check and execute the onEnd parameter
     *
     * @param kwix
     */
    var doAfterThings = function( kwix ){
        kwix.one('webkitAnimationEnd mozAnimationEnd MSAnimationEnd oanimationend animationend', function(){
            //get current effect onEnd property
            var onEnd = effects[effect]['onEnd'];

            //
            switch(onEnd){
                case 'hide':
                    kwix.hide();
                    break;
                default:
                    if(typeof onEnd == 'function'){
                        onEnd.call(this);
                    }
                    break;
            }
        });
    }
    /**
     * Define the plugin
     *
     * @param options
     * @returns {jQuery}
     */
    $.fn.animakWix = function(options) {
        //check if user defined a delay for animation
        doDelayThings(options);
        //set the animation on fire
        kwix = this;
        setTimeout(function(){
            doTheChecks(options);
            doBeforeThings(kwix);
            doTheThings(kwix);
            doAfterThings(kwix);
        },delay);
        return this;
    };

}( jQuery ));
